﻿#region License
/* NBit Bit Manipulation Library
   Copyright 2012 Illusory Studios LLC <james@illusorystudios.com>

   This software is provided 'as-is', without any express or implied
   warranty. In no event will the authors be held liable for any damages
   arising from the use of this software.
 
   Permission is granted to anyone to use this software for any purpose,
   including commercial applications, and to alter it and redistribute it
   freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must
   not claim that you wrote the original software. If you use this software
   in a product, an acknowledgement in the product documentation would be
   appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must
   not be misrepresented as being the original software.

   3. This notice may not be removed or altered from any source
   distribution. 
*/
#endregion

using NUnit.Framework;

namespace NBit.Tests
{
    [TestFixture]
    public class BitMathExtensionsTest
    {
        [Test]
        public void CountBits()
        {
            Assert.AreEqual(0, (0x00000000u).CountBits());
            Assert.AreEqual(1, (0x80000000u).CountBits());
            Assert.AreEqual(1, (0x00000001u).CountBits());
            Assert.AreEqual(32, (0xffffffffu).CountBits());
        }

        [Test]
        public void CountTrailingZeros()
        {
            Assert.AreEqual(32, (0x00000000u).CountTrailingZeros());
            Assert.AreEqual(31, (0x80000000u).CountTrailingZeros());
            Assert.AreEqual(0, (0x00000001u).CountTrailingZeros());
            Assert.AreEqual(0, (0xffffffffu).CountTrailingZeros());
        }

        [Test]
        public void IsOddParity()
        {
            Assert.IsTrue((4u).IsOddParity());
            Assert.IsTrue((42u).IsOddParity());
            Assert.IsFalse((0u).IsOddParity());
            Assert.IsFalse((40u).IsOddParity());
        }

        [Test]
        public void IsPowerOf2()
        {
            Assert.IsTrue((0x80000000u).IsPowerOf2());
            Assert.IsTrue((2u).IsPowerOf2());
            Assert.IsTrue((1u).IsPowerOf2());
            Assert.IsFalse((0xffffffffu).IsPowerOf2());
            Assert.IsFalse((3u).IsPowerOf2());
            Assert.IsFalse((0u).IsPowerOf2());
        }

        [Test]
        public void ReverseBits()
        {
            Assert.AreEqual(0xedb88320, (0x04c11db7u).ReverseBits());
        }

        [Test]
        public void RoundUpToPowerOf2()
        {
            Assert.AreEqual(0x00000000u, (0xffffffffu).RoundUpToPowerOf2());
            Assert.AreEqual(0x00000000u, (0x80000002u).RoundUpToPowerOf2());
            Assert.AreEqual(0x00000000u, (0x80000001u).RoundUpToPowerOf2());
            Assert.AreEqual(0x80000000u, (0x80000000u).RoundUpToPowerOf2());
            Assert.AreEqual(0x80000000u, (0x7fffffffu).RoundUpToPowerOf2());
            Assert.AreEqual(4, (3u).RoundUpToPowerOf2());
            Assert.AreEqual(2, (2u).RoundUpToPowerOf2());
            Assert.AreEqual(1, (1u).RoundUpToPowerOf2());
            Assert.AreEqual(1, (0u).RoundUpToPowerOf2());
        }
    }
}

