﻿#region License
/* NBit Bit Manipulation Library
   Copyright 2012 Illusory Studios LLC <james@illusorystudios.com>

   This software is provided 'as-is', without any express or implied
   warranty. In no event will the authors be held liable for any damages
   arising from the use of this software.
 
   Permission is granted to anyone to use this software for any purpose,
   including commercial applications, and to alter it and redistribute it
   freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must
   not claim that you wrote the original software. If you use this software
   in a product, an acknowledgement in the product documentation would be
   appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must
   not be misrepresented as being the original software.

   3. This notice may not be removed or altered from any source
   distribution. 
*/
#endregion

namespace NBit
{
    public abstract partial class Crc
    {
        uint _preset, _postinvert;

        internal Crc(uint preset, uint postinvert)
        {
            _preset = preset; _postinvert = postinvert;
        }

        internal virtual void Init()
        {

        }

        /// <summary>
        /// Returns the initial value of the CRC. Begin the computation using this value.
        /// </summary>
        /// <returns>The initial value of the CRC.</returns>
        public uint Begin()
        {
            return _preset;
        }

        /// <summary>
        /// Sends 8 bits through the CRC.
        /// </summary>
        /// <param name="crc">The current CRC value.</param>
        /// <param name="byte">The byte to send through the CRC.</param>
        /// <returns>The modified CRC value.</returns>
        public virtual uint Next(uint crc, byte @byte)
        {
            return NextBits(crc, @byte, 8);
        }

        /// <summary>
        /// Sends a specified number of bits through the CRC.
        /// </summary>
        /// <param name="crc">The current CRC value.</param>
        /// <param name="byte">The bits to send through the CRC.</param>
        /// <param name="bits">The number of bits to send.</param>
        /// <returns>The modified CRC value.</returns>
        public uint Next(uint crc, uint @byte, int bits)
        {
            return bits == 8 ? Next(crc, (byte)@byte) : NextBits(crc, @byte, bits);
        }

        internal abstract uint NextBits(uint crc, uint @byte, int bits);

        /// <summary>
        /// Undoes the End operation. An exclusive OR is done between the final CRC value and the post-invert bit mask.
        /// </summary>
        /// <param name="crc">The final CRC value.</param>
        /// <returns>The continuable CRC value.</returns>
        public uint Resume(uint crc)
        {
            return End(crc);
        }

        /// <summary>
        /// Finishes the CRC computation. An exclusive OR is done between the current CRC value and the post-invert bit mask.
        /// </summary>
        /// <param name="crc">The current CRC value.</param>
        /// <returns>The final CRC value.</returns>
        public uint End(uint crc)
        {
            return crc ^ _postinvert;
        }
    }
}
