﻿#region License
/* NBit Bit Manipulation Library
   Copyright 2012 Illusory Studios LLC <james@illusorystudios.com>

   This software is provided 'as-is', without any express or implied
   warranty. In no event will the authors be held liable for any damages
   arising from the use of this software.
 
   Permission is granted to anyone to use this software for any purpose,
   including commercial applications, and to alter it and redistribute it
   freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must
   not claim that you wrote the original software. If you use this software
   in a product, an acknowledgement in the product documentation would be
   appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must
   not be misrepresented as being the original software.

   3. This notice may not be removed or altered from any source
   distribution. 
*/
#endregion

using System;

namespace NBit
{
    /// <summary>
    /// Specifies the format of the CRC.
    /// </summary>
    [Flags]
    public enum CrcFormat
    {
        /// <summary>
        /// The least significant bit of each byte is sent through the CRC first.
        /// </summary>
        LsbFirst = 1,

        /// <summary>
        /// The most significant bit of each byte is sent through the CRC first.
        /// </summary>
        MsbFirst = 2,

        /// <summary>
        /// The constant term and the highest order polynomial term are always one for CRCs.
        /// A 16-bit CRC, for example, will always have x^16 and x^0 terms. It's a matter
        /// of convention which is left out, and more commonly it is the highest-order term.
        /// If the polynomial is in Koopman notation, the lowest-order term is dropped instead.
        /// This corresponds to "Reversed Reciprocal" entries on Wikipedia's CRC page.
        /// </summary>
        KoopmanPolynomial = 4,

        /// <summary>
        /// The order of bits in the CRC polynomial is reversed.
        /// Higher order terms correspond to less significant bits.
        /// This corresponds to "Reversed" entries on Wikipedia's CRC page.
        /// </summary>
        ReversedPolynomial = 8
    }
}
