﻿#region License
/* NBit Bit Manipulation Library
   Copyright 2012 Illusory Studios LLC <james@illusorystudios.com>

   This software is provided 'as-is', without any express or implied
   warranty. In no event will the authors be held liable for any damages
   arising from the use of this software.
 
   Permission is granted to anyone to use this software for any purpose,
   including commercial applications, and to alter it and redistribute it
   freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must
   not claim that you wrote the original software. If you use this software
   in a product, an acknowledgement in the product documentation would be
   appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must
   not be misrepresented as being the original software.

   3. This notice may not be removed or altered from any source
   distribution. 
*/
#endregion

namespace NBit
{
    sealed class CrcLsb : Crc
    {
        uint _polynomial; uint[] _table;

        public CrcLsb(uint polynomial, uint preset, uint postinvert)
            : base(preset, postinvert)
        {
            _polynomial = polynomial;
        }

        internal override void Init()
        {
            base.Init();
            _table = new uint[256];
            for (int i = 0; i < _table.Length; i++) { _table[i] = NextBits(0, (uint)i, 8); }
        }

        public override uint Next(uint crc, byte @byte)
        {
            return (crc >> 8) ^ _table[@byte ^ (crc & 0xff)];
        }

        internal override uint NextBits(uint crc, uint @byte, int bits)
        {
            var polynomial = _polynomial;

            if (bits < 32) { @byte &= (1u << bits) - 1; }
            crc ^= @byte;

            for (int i = 0; i < bits; i++)
            {
                if (0 != (crc & 1))
                {
                    crc >>= 1; crc ^= polynomial;
                }
                else
                {
                    crc >>= 1;
                }
            }

            return crc;
        }
    }
}
